/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

/****************************************************************************
** $Id: application.cpp,v 1.110 2004/09/02 10:24:12 flitney Exp $
**
** Copyright (C) 2002 University of Oxford.  All rights reserved.
**
** FSLView
**
*****************************************************************************/
#include "viewwidget.h"
#include "orthowidget.h"
#include "lightboxwidget.h"
#include "singlewidget.h"
#include "timeserieswidget.h"
#include "histogramwidget.h"
#include "application.h"
#include <stdio.h>
#include <qworkspace.h>
#include <qpixmap.h>
#include <qlabel.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qaccel.h>
#include <qwhatsthis.h>
#include <qtoolbutton.h>
#include <qtooltip.h>

#include "fileopen.xpm"
#include "fsllogo.xpm"

#include "tracker.h"
#include "version.h"
#include "storage/error.h"
#include "storage/image.h"
#include "avwio/avwio.h"

#include "modelfit.h"

const char * fileOpenText = "Click this button to open a <em>folder</em>.<br><hr>"
"You can also select the <b>Open command</b> from the File menu.";

ApplicationWindow::ApplicationWindow()
  : QMainWindow( 0, "FSLView", WDestructiveClose )
{  
  TRACKER("ApplicationWindow::ApplicationWindow");
  qApp->setMainWidget(this);
  setIcon( QPixmap(fsllogo_xpm) );
  setupStatusBar(); 

  m_ws = new QWorkspace(this);
  m_ws->setBackgroundMode( PaletteMidlight );

  #if (QT_VERSION >= 300)
	m_ws->setScrollBarsEnabled(true); //QT 3 only
  #endif

  setCentralWidget(m_ws);
  buildMenus(); 
  connectControls(); 
  show();
  
  emit message( "Ready", 2000 ); 
  

  if(qApp->argc() > 1)
    {
      QString arg(qApp->argv()[1]);
      QFileInfo  fi(qApp->argv()[1]);
      loadFile(fi.absFilePath());
      QDir::setCurrent(fi.dirPath(true));
    }
 }



ApplicationWindow::~ApplicationWindow()
{
  TRACKER("ApplicationWindow::~ApplicationWindow");
}


void ApplicationWindow::load()
{
 QString fn = QFileDialog::getOpenFileName( QString::null, "Analyze files (*.hdr *.hdr.gz)", this );

 setCurrentDir(fn);
 if(!fn.isEmpty())loadFile(fn);
}

void ApplicationWindow::loadFile(const QString & absFilePath)
{ 
  QString fn(absFilePath);
  QFileInfo fi(absFilePath);
  bool useGz(false);

  if(fi.extension(false) == "gz")useGz = true;


  removeExtensions(fn);

  try
  {
    if ( !fn.isEmpty() && checkFilesExist(fn,false)) 
      {
      emit message( QString("Loading.... %1").arg(fn), 0);



      m_imageGroup = ImageGroup::create( Image::load((const char *)fn,useGz) );
      ImageInfo::Handle info(m_imageGroup->getMainImage()->getInfo());
      m_cursor = Cursor::create(this,info->inqX(),
                                info->inqY(),info->inqZ(), 
                                info->inqNumVolumes());

      orthoView();
   
      setCaption( fn );

      m_cursor->setCursor( info->inqX()/2, info->inqY()/2, info->inqZ()/2 ); 

      QApplication::restoreOverrideCursor();      

      if(checkForDuplicates(fn))
        {
          QMessageBox::warning(this,"FSLView",
          "Warning: Zipped and Unzipped versions of the data exist!");
        }

      }  
    else 
      {
        if(!checkFilesExist(fn,false))
          {
            emit message( QString("Could not open %1").arg(fn), 2000 );
            if(!checkFilesExist(fn,true))
              {
                emit message( QString("Missing image(.img) file %1").arg(fn), 2000 );
              }
          }
        else
          {
            emit message( "Loading aborted", 2000 );
          }
      }
  }
  catch(FileError& f)
    {
      catchFileError(f);
    }  
}

void ApplicationWindow::closeWindow()
{
  QWidget* w = m_ws->activeWindow();
  if(w){w->close();}
}

bool ApplicationWindow::windowListEmpty()
{
  return m_ws->windowList().isEmpty();
}

int ApplicationWindow::windowListCount()
{
  return m_ws->windowList().count();
}

void ApplicationWindow::about()
{
  
  QMessageBox::about( this, "FSLView",
		      QString("<h2>FSLView</h2><hr><b>Version %1.%2</b><br>"
			      "Written by Dave Flitney, James Saunders and Steve Smith<br>"
			      "Copyright(c) 2004 University of Oxford<br><hr>"
            "Help pages:<br>"
			      "<b>html://www.fmrib.ox.ac.uk/fsl/fslview</b><hr>"
			      "Please report  bugs to:<br><b>fslview-bugs@fmrib.ox.ac.uk<b>")
		      .arg(Version).arg(Release));

}



void ApplicationWindow::aboutQt()
{
  QMessageBox::aboutQt( this, "fslview" );
}

void ApplicationWindow::windowsMenuAboutToShow()
{ 
  m_windowsMenu->clear();
  int cascadeId = m_windowsMenu->insertItem("&Cascade", m_ws, SLOT(cascade() ) );
  int tileId = m_windowsMenu->insertItem("&Tile", m_ws, SLOT(tile() ) );
  if ( m_ws->windowList().isEmpty() ) {
    m_windowsMenu->setItemEnabled( cascadeId, FALSE );
    m_windowsMenu->setItemEnabled( tileId, FALSE );
  }
  m_windowsMenu->insertSeparator();
  QWidgetList windows = m_ws->windowList();
  for ( int i = 0; i < int(windows.count()); ++i ) {
    int id = m_windowsMenu->insertItem(windows.at(i)->caption(),
				      this, SLOT( windowsMenuActivated( int ) ) );
    m_windowsMenu->setItemParameter( id, i );
    m_windowsMenu->setItemChecked( id, m_ws->activeWindow() == windows.at(i) );
  }
}

void ApplicationWindow::windowsMenuActivated( int id )
{
  QWidget* w = m_ws->windowList().at( id );
  if ( w ) {
    w->showNormal();
    w->setFocus();
  }
}
void ApplicationWindow::fileMenuAboutToShow()
{
  bool remEnabled(false);
  bool state(windowListEmpty());  
  ViewWidget *view = (ViewWidget*)m_ws->activeWindow();

  if (!state && view)
    {
      OverlayList::Handle ol = view->getOverlayList();
      if(isValidOverlayList(ol))
        {
          if(ol->getActiveMetaImage().get())
            remEnabled = !ol->getActiveMetaImage()->getInfo()->isMainImage();
        }
    }

  m_fileMenu->setItemEnabled(m_fileCreateMaskID,!state);
  m_fileMenu->setItemEnabled(m_fileAddOverlayID,!state);
  m_fileMenu->setItemEnabled(m_fileRemOverlayID,remEnabled );
  m_fileMenu->setItemEnabled(m_fileSaveOverlayID,!state );
  m_fileMenu->setItemEnabled(m_fileOpenID,state);
  m_fileMenu->setItemEnabled(m_fileCloseID,!state); 
}

void ApplicationWindow::viewMenuAboutToShow()
{  
  TRACKER("ApplicationWindow::viewMenuAboutToShow");
  bool state(!windowListEmpty());      
  bool multiVolume(false); 
  bool validImage(false);
  
  ViewWidget *view = (ViewWidget*)m_ws->activeWindow();

  if(view)
    {
      OverlayList::Handle ol = view->getOverlayList();
      if(isValidOverlayList(ol))
	{
	  Image::Handle image = ol->inqActiveImage();
	  if(isValidImage(image))
	    {
	      validImage= true;
	      multiVolume = (image->getInfo()->inqNumVolumes()>1);
	    }
	}
    }
  else
    state = false;

  m_viewMenu->setItemEnabled(m_viewOrthoID,state);
  m_viewMenu->setItemEnabled(m_viewLightID,state);  
  m_viewMenu->setItemEnabled(m_viewSingleID, state);   
  m_viewMenu->setItemEnabled(m_viewHistID,state && validImage);  
  m_viewMenu->setItemEnabled(m_viewTimeID,state && multiVolume && validImage);  
}

void ApplicationWindow::setupStatusBar()
{
  m_statusBar = statusBar();
  m_statusBar->addWidget(new QLabel(m_statusBar), 1, FALSE);
}

void ApplicationWindow::buildMenus()
{
  m_mb = menuBar();
  
  QPixmap openIcon = QPixmap( fileopen_xpm );

  m_fileMenu = new QPopupMenu( this );
  connect( m_fileMenu, SIGNAL( aboutToShow() ),
	   this, SLOT( fileMenuAboutToShow() ) );
  m_fileOpenID        = m_fileMenu->insertItem( openIcon, "&Open...", this, SLOT(load()), CTRL+SHIFT+Key_O );   
  m_fileOpenID        = m_fileMenu->insertItem( openIcon, "&Open Feat...", this, SLOT(loadFeat()), CTRL+SHIFT+Key_F ); 
  m_fileCreateMaskID  = m_fileMenu->insertItem( "Create &mask", this, SLOT(createMask()), CTRL+Key_C );
  m_fileSaveOverlayID = m_fileMenu->insertItem( "&Save as...",   this, SLOT(saveOverlay()), CTRL+Key_S );
  m_fileAddOverlayID  = m_fileMenu->insertItem( "&Add...",    this, SLOT(addOverlay()), CTRL+Key_A );   
  m_fileRemOverlayID  = m_fileMenu->insertItem( "&Remove", this, SLOT(remOverlay()), CTRL+Key_R );
  m_fileMenu->insertSeparator();
  m_fileCloseID       = m_fileMenu->insertItem( "&Close", this, SLOT(closeWindow()), CTRL+Key_W );
  
  m_fileMenu->insertItem( "&Quit", qApp, SLOT( closeAllWindows() ), CTRL+Key_Q );
  
  m_windowsMenu = new QPopupMenu( this );
  connect( m_windowsMenu, SIGNAL( aboutToShow() ),
	   this, SLOT( windowsMenuAboutToShow() ) );
  m_windowsMenu->setCheckable( TRUE );
  
  m_viewMenu = new QPopupMenu( this );
  connect( m_viewMenu, SIGNAL( aboutToShow() ),
	   this, SLOT( viewMenuAboutToShow() ) );
  m_viewMenu->setCheckable(true);
  m_viewOrthoID = m_viewMenu->insertItem( "&Orthographic", this, SLOT(orthoView()), CTRL+Key_O );
  m_viewLightID = m_viewMenu->insertItem( "&Lightbox", this, SLOT(lightboxView()), CTRL+Key_L );
  m_viewSingleID = m_viewMenu->insertItem( "&Single", this, SLOT(singleView()), CTRL+Key_M );
  m_viewMenu->insertSeparator();
  m_viewHistID = m_viewMenu->insertItem( "Image &histogram", this, SLOT(openHistogram()), CTRL+Key_H );
  m_viewTimeID = m_viewMenu->insertItem( "&Timeseries", this, SLOT(openTimeSeries()), CTRL+Key_T );

  QPopupMenu * help = new QPopupMenu( this );

  help->insertItem( "&About FSLView", this, SLOT(about()), Key_F1);
  help->insertSeparator();
  help->insertItem( "About&Qt", this, SLOT(aboutQt()));
  
  m_mb->insertItem( "&File", m_fileMenu );  
  m_mb->insertItem( "&Windows", m_windowsMenu );
  m_mb->insertItem( "&View",m_viewMenu  );
  m_mb->insertSeparator();
  m_mb->insertItem( "&Help", help );
}

void ApplicationWindow::connectControls()
{  
  connect( this, SIGNAL(      message(const QString&, int)),
           this, SLOT( displayMessage(const QString&, int)));
}

void ApplicationWindow::update(const Cursor::Handle& c)
{
}

void ApplicationWindow::saveOverlay()
{
	
  
  ViewWidget *view = (ViewWidget*)m_ws->activeWindow();
  OverlayList::Handle ol = view->getOverlayList();
  QString fn;
	
  #if !defined(WIN32)
	bool useGz = bool(usecompression());
  #else
	bool useGz = true;
  #endif

	if(!isValidOverlayList(ol))
    {
      QMessageBox::warning(this, "FSLView",
			   "Active window must be an OrthoView, Lightbox or Single View");
      return;
    }

  Image::Handle image = ol->inqActiveImage();
    
  if(!isValidImage(image))
        {
          QMessageBox::warning(this,"FSLView",
			       "No image selected in Active window");
          return;
        }
   
          // Okay can go ahead and try to save it then
  QString initFileName;
  if(useGz){initFileName = QString(image->getInfo()->inqImageName().c_str()) + ".hdr.gz";}
  else {initFileName = QString(image->getInfo()->inqImageName().c_str()) + ".hdr";}
  

  fn = QFileDialog::getSaveFileName(initFileName, 
                                    "Analyze files (*.hdr *.hdr.gz)", this,
                                    "save file dialog",
                                    "Select a filename for saving");
  	

  QFileInfo fi(fn);
  if(fi.extension(false) == "gz"){useGz = true;}
  else                           {useGz = false;}
 
 setCurrentDir(fn);    
 removeExtensions(fn);

 if(checkSpecificFilesExist(fn,useGz))   
    {
      if(QMessageBox::warning( this,
                               "FSLView",
                               "File already exists. Do you want to overwrite it?",
                               "Cancel","OK","",1,0) == 0) 
              {      
                return;
              }
    }
     

   
               if ( !fn.isEmpty() ) 
                 {
                 if(image->save((const char *)fn, useGz))
                   {
                     image->getInfo()->setTarnished(false);  
                     if(checkForDuplicates(fn))
                      {
                        QMessageBox::warning(this,"FSLView",
                        "Warning: Zipped and Unzipped versions of the data exist!");
                      }
                   }
                 else
                   {
                     QMessageBox::warning(this, "FSLView",
                                          QString("<h2>Save failed!</h2>") +
                                          "<p>Perhaps you're out of disk space or " +
                                          "you don't have permission to write to " +
                                          "this directory." +
                                          "<p>Please see console for more details." +
                                          "<p>Click Ok and try again.",
                                          QMessageBox::Ok | QMessageBox::Default | QMessageBox::Escape,
                                          QMessageBox::NoButton);
                   }
                 }
  

  }





void ApplicationWindow::remOverlay()
{  
  
 ViewWidget* view = (ViewWidget*)m_ws->activeWindow();
 OverlayList::Handle ol = view->getOverlayList();
 if(!isValidOverlayList(ol))
   {
     QMessageBox::warning(this, "FSLView",
                          "Active window must be either OrthoView, Lightbox or Single View");
   }
 else
  {
        Image::Handle mainImg = ol->getMainImage();
    Image::Handle image = ol->inqActiveImage();
    
    if(!isValidImage(image))
      {
        QMessageBox::warning(this,
                             "FSLView",
                             "No image selected in Active window");
      }
    else if(mainImg == image)
      {
        QMessageBox::warning(this,"FSLView","Cannot remove main image");
      }
    else
      {
        if(tarnishCheck(image)){ m_imageGroup->remOverlay(image);}
      }
  }
}


void ApplicationWindow::addOverlay()
{ 

  try{
    
    QString fn = QFileDialog::getOpenFileName( QString::null, "Analyze files (*.hdr *.hdr.gz)", this );
    QFileInfo fi(fn);
    bool useGz(false);
    if(fi.extension(false) == "gz")useGz = true;
    
    setCurrentDir(fn);
    removeExtensions(fn);
    if ( !fn.isEmpty() && checkFilesExist(fn,false) ) 
      {
        QApplication::setOverrideCursor(Qt::waitCursor);
        emit message( QString("Loading.... %1").arg(fn), 0);



        Image::Handle overlay = Image::load((const char *)fn,true);
        QApplication::restoreOverrideCursor();
        


        if(m_imageGroup->getMainImage()->getInfo()->isCompatible(overlay->getInfo()))
          {
            m_imageGroup->addOverlay( overlay );
          }
        else
          {
            QMessageBox::warning( this, "FSLView",
                                  "Cannot load incompatible overlay" );
            emit message( "Loading aborted", 2000 );
          }        
        
        if(checkForDuplicates(fn))
        {
          QMessageBox::warning(this,"FSLView",
          "Warning: Zipped and Unzipped versions of the data exist!");
        }
      } 
    else 
      {        
        if(!checkFilesExist(fn,false))
          {
                       emit message( QString("Could not open %1").arg(fn), 2000 );            
                       if(!checkFilesExist(fn,true))
                         {
                           emit message( QString("Missing image(.img) file %1").arg(fn), 2000 );
                         }
          }
        else
          {
            emit message( "Loading aborted", 2000 );
          }
      }
  }
   catch(FileError& f)
   {
     catchFileError(f);
   }  

}

void ApplicationWindow::addLookUpTable()
{
  QString fn = QFileDialog::getOpenFileName( QString::null, "LUTs (*.lut *.rgb)", this );
  if ( !fn.isEmpty() ) {
    QApplication::setOverrideCursor(Qt::waitCursor);
    emit message( QString("Loading lookup table.... %1").arg(fn), 2000 );
    LookUpTable::Handle lookUpTable = LookUpTable::load((const char *)fn);
    m_imageGroup->addLookUpTable( lookUpTable );

    QApplication::restoreOverrideCursor();

 }  else {
    emit message( "Loading aborted", 2000 );
  }
}


void ApplicationWindow::createMask()
{
 TRACKER("ApplicationWindow::createMask");      
 ViewWidget* view = (ViewWidget*)m_ws->activeWindow();
 OverlayList::Handle ol = view->getOverlayList();
 if(!isValidOverlayList(ol))
   {
     QMessageBox::warning(this,
                          "FSLView",
                          "Active window must be either OrthoView, Lightbox or Single View");
   }
 else
  {
    
    Image::Handle mainImg = ol->getMainImage();
    Image::Handle image = ol->inqActiveImage();
    
    if(!isValidImage(image))
      {
        QMessageBox::warning(this,
                             "FSLView",
                             "No image selected in Active window");
      }
    else
      {
        Image::Handle spawn = image->cloneStructure();
        m_imageGroup->addOverlay(spawn);
        emit message( QString("Created new mask: %1-mask")
		      .arg(image->getInfo()->inqImageName().c_str()), 2000 );
      }
  }

}

void ApplicationWindow::orthoView()
{  
  ViewWidget* view = new OrthoWidget(m_ws,m_imageGroup,
                                     copyActiveOverlayList(), m_cursor); 

  connect( view, SIGNAL(message(const QString&, int)), 
           this, SIGNAL(message(const QString&, int)) );
  connect( view, SIGNAL(addLookUpTable()),
           this, SLOT(addLookUpTable()));
  connect( view, SIGNAL(windowClose(QCloseEvent*)),
           this, SLOT(childWindowClose(QCloseEvent*)));
  view->setCaption("Ortho view");

  view->resize(580,500);

  viewShow(view);
}

void ApplicationWindow::lightboxView()
{
  ViewWidget* view = new LightboxWidget(m_ws, m_imageGroup,
                                        copyActiveOverlayList(),m_cursor); 

  connect( view, SIGNAL(message(const QString&, int)), 
           this, SIGNAL(message(const QString&, int)) );   
  connect( view, SIGNAL(addLookUpTable()),
           this, SLOT(addLookUpTable()));  
  connect( view, SIGNAL(windowClose(QCloseEvent*)),
           this, SLOT(childWindowClose(QCloseEvent*)));
  view->setCaption("Lightbox view");  
  view->resize(580,500);
  viewShow(view);
}

void ApplicationWindow::singleView()
{
  ViewWidget* view = new SingleWidget(m_ws, m_imageGroup,
                                     copyActiveOverlayList(),
                                     m_cursor); 

  connect( view, SIGNAL(message(const QString&, int)), 
           this, SIGNAL(message(const QString&, int)) );   
  connect( view, SIGNAL(addLookUpTable()),
           this, SLOT(addLookUpTable()));  
  connect( view, SIGNAL(windowClose(QCloseEvent*)),
           this, SLOT(childWindowClose(QCloseEvent*)));
  view->setCaption("Single view");  
  view->resize(210,345);
  viewShow(view);  
}

void ApplicationWindow::openTimeSeries()
{
  if(m_imageGroup.use_count())
  {  
    ViewWidget *view = (ViewWidget*)m_ws->activeWindow();
    OverlayList::Handle ol = view->getOverlayList();
    
    if(isValidOverlayList(ol))
    {
      Image::Handle image = ol->inqActiveImage();
    
      if(isValidImage(image))
        {
          if(image->getAvw() == NULL)
            {
              QMessageBox::warning( this,
                                    "FSLView",
                                    "New masks must be edited, saved and reloaded before viewing their timeseries. This is a known error.");
            }
          else
            {
              TimeSeriesWidget* timeseries = new TimeSeriesWidget(m_ws, image, m_cursor);  
              connect( timeseries, SIGNAL(windowClose(QCloseEvent*)),
                       this, SLOT(childWindowClose(QCloseEvent*)));
              timeseries->setCaption("Timeseries");    
              timeseries->resize(250,250);
              viewShow(timeseries);
            }
        }
    }
  }
}


void ApplicationWindow::openHistogram()
{
  if(m_imageGroup.use_count())
  {  
    ViewWidget *view = (ViewWidget*)m_ws->activeWindow();
    OverlayList::Handle ol = (view != NULL) ? view->getOverlayList() : OverlayList::Handle();
    
    if(isValidOverlayList(ol))
    {
      Image::Handle image = ol->inqActiveImage();
    
      if(isValidImage(image))
        {
	  unsigned int v = m_cursor->inqV();
	  HistogramWidget* histogram = new HistogramWidget(m_ws, image->getVolume(v),
							   image->getInfo()->inqImageName(),
							   v,
							   image->getInfo()->isInteger());   
          connect( histogram, SIGNAL(windowClose(QCloseEvent*)),
                   this, SLOT(childWindowClose(QCloseEvent*))); 
          viewShow(histogram);
        }
    }
  }
}  

void ApplicationWindow::catchFileError(FileError f)
{
  emit message( QString("Error loading %1, %2")
                        .arg(f.inqFileName().c_str())
                        .arg(f.inqMessage().c_str()),
                         3000 ); 
}

bool ApplicationWindow::checkAbsFilePath(const QString & absFilePath,
                                         QString ext)
{
  return QFile::exists(absFilePath + "." + ext);
}

OverlayList::Handle ApplicationWindow::activeOverlayList()
{
  OverlayList::Handle ol;
  ViewWidget* view = (ViewWidget*)m_ws->activeWindow();

  if (!windowListEmpty())ol = view->getOverlayList();
  return ol;
}

OverlayList::Handle ApplicationWindow::copyActiveOverlayList()
{
  OverlayList::Handle null;
  OverlayList::Handle ol = activeOverlayList();
  
  if(isValidOverlayList(ol)){return ol->clone();}
  else                      {return null;}
}
 
void ApplicationWindow::displayMessage(const QString & msg, int time)
{
  statusBar()->message( msg, time );
}

void ApplicationWindow::viewShow(ViewWidget* v)
{
  if(windowListEmpty())
    { 
      v->showMaximized();
    }
  else if(m_ws->windowList().count() == 1) 
    {
      m_ws->windowList().at(0)->showNormal();
      v->showNormal();
      m_ws->tile();
    } 
  else
    {
      v->show();
    }
}

bool ApplicationWindow::tarnishCheck(Image::Handle& image)
{
  bool result(true);

  if(image->getInfo()->inqTarnished())
    {
        QString imageMessage = QString("Continuing this action will lose changes to %1?")
          .arg(image->getInfo()->inqImageName().c_str());
        
        switch(QMessageBox::warning( this,
                               "FSLView",
                               imageMessage,"Cancel","OK","",0,0))
          {
          case 0:result = false;break;
          case 1:result = true ;break;
          }
    }
  
  return result;
}

bool ApplicationWindow::tarnishCheck()
{

 bool result(true);

  if(m_imageGroup->inqTarnished())
    {
        QString imageMessage = QString("Continuing this action will lose unsaved data.");
        switch(QMessageBox::warning( this,
                               "FSLView",
                               imageMessage,"Cancel","OK","",0,0))
          {
          case 0:result = false;break;
          case 1:result = true ;break;
          }
    }
  
  return result;
}

void ApplicationWindow::childWindowClose(QCloseEvent* e)
{
  if(windowListCount() > 1)
    {
      e->accept();
    }
  else
    {
      if(tarnishCheck())e->accept();
    }
  
}


void ApplicationWindow::closeEvent(QCloseEvent* e)
{
  if(windowListEmpty())
    {
      e->accept();
    }
  else
    {
      if(tarnishCheck())e->accept();
    }
}
  
void ApplicationWindow::setCurrentDir(QString path)
{
  QFileInfo fi(path);
  QDir::setCurrent(fi.dirPath(true));
}

void ApplicationWindow::removeExtensions(QString & fileName)
{
  //Strips off the gz and then the hdr and img extensions

  QFileInfo fiA(fileName);
  if(fiA.extension(false) == "gz")  
    fileName.remove( fileName.findRev("."), 3 );

  QFileInfo fiB(fileName);
  if((fiB.extension(false) == "img") || (fiB.extension(false) == "hdr"))
    fileName.remove( fileName.findRev("."), 4 );
}
  

bool ApplicationWindow::checkFilesExist(const QString & fn, bool justImg)
{
  //Checks that either hdr.gz and img.gz or  hdr and img files exist
  bool filesExist(true);
  
  if(!justImg)
    {
      if(!checkAbsFilePath(fn,"hdr") &&
         !checkAbsFilePath(fn,"hdr.gz") )filesExist = false;
    }
  
  if(!checkAbsFilePath(fn,"img") &&
     !checkAbsFilePath(fn,"img.gz") )filesExist = false;

  return filesExist;
}
  

bool ApplicationWindow::checkSpecificFilesExist(const QString & fn, bool useGz)
{
  bool filesExist(false);
  
  if(useGz)
    {
      if(checkAbsFilePath(fn,"hdr.gz") ||
         checkAbsFilePath(fn,"img.gz") )filesExist = true;
    }
  else
    {
      if(checkAbsFilePath(fn,"hdr") ||
         checkAbsFilePath(fn,"img") )filesExist = true;
    }

  return filesExist;
}

bool ApplicationWindow::checkForDuplicates(const QString & fn)
{
  bool duplicatesExist(false);

  if(checkAbsFilePath(fn,"hdr") && checkAbsFilePath(fn,"hdr.gz") )
    duplicatesExist = true;

  if(checkAbsFilePath(fn,"img") && checkAbsFilePath(fn,"img.gz") )
    duplicatesExist = true;

  return duplicatesExist;
}

void ApplicationWindow::loadFeat()
{
  QString dirName,fileName;    
  QFileDialog* fd = new QFileDialog( this, "file dialog", TRUE );
  fd->setMode( QFileDialog::DirectoryOnly );
  fd->setCaption("Select Feat directory");

  if ( fd->exec() == QDialog::Accepted )
    dirName = fd->selectedFile();
  
  fileName = dirName + "filtered_func_data";

  QFileInfo  fi(fileName);
  loadFile(fi.absFilePath());       

  ModelFit::Handle modelFit = ModelFit::create(fi.dirPath(true));

  Image::Handle image = m_imageGroup->getMainImage();

  TimeSeriesWidget* timeseries = new TimeSeriesWidget(m_ws, image, 
                                                        m_cursor,modelFit);  
  connect( timeseries, SIGNAL(windowClose(QCloseEvent*)),
           this, SLOT(childWindowClose(QCloseEvent*)));
  timeseries->setCaption("Feat data");    
  timeseries->resize(250,250);
  viewShow(timeseries);
}
