/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */


#include "briconwidget.h"

#include <qlineedit.h>
#include <qvalidator.h>
#include <qtoolbutton.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qlabel.h>
#include <qtooltip.h>

#include <qwt_wheel.h>

#include <algorithm>
#include <stdio.h>

#include "tracker.h"

#include "bricon.xpm"

class BriConWheel: public QwtWheel
{
public:
  BriConWheel(QWidget *parent, const char *name = 0): QwtWheel(parent, name)
  {
    setOrientation(Qt::Horizontal);
    setRange(-49, 49, 2, 1);
    setTickCnt(20);
    setViewAngle(170);
    setTotalAngle(6 * 360);
  }
};

BriConWidget::BriConWidget(QWidget* w, OverlayList::Handle list) : 
  QWidget(w), m_list(list), m_blockEvents(false)
{
  MetaImage::Handle mi = m_list->getActiveMetaImage();
  if(!mi.get())
    mi = m_list->getMainMetaImage();
  m_bricon = mi->getDs()->inqBriCon();

  m_list->attach(this);



  m_minBox = new QLineEdit(this);
  m_minBox->setMinimumWidth( QFontMetrics(m_minBox->font()).width(QString("-0.000000e-00")) );
  m_minBox->setMaximumWidth( QFontMetrics(m_minBox->font()).width(QString("-0.000000e-00")) );
  m_minBox->setText( QString("%1").arg(m_bricon->inqMin(),7,'g',5) );
  m_minBox->setValidator(new QDoubleValidator(this));







  m_maxBox = new QLineEdit(this);
  m_maxBox->setMinimumWidth( QFontMetrics(m_minBox->font()).width(QString("-0.000000e-00")) );
  m_maxBox->setMaximumWidth( QFontMetrics(m_maxBox->font()).width(QString("-0.000000e-00")) );

  m_maxBox->setText( QString("%1").arg(m_bricon->inqMax(),7,'g',5) );
  m_maxBox->setValidator(new QDoubleValidator(this));

  m_minBox->setAlignment(AlignLeft);
  m_maxBox->setAlignment(AlignLeft);

  m_briSlider  = new BriConWheel(this);
  m_conSlider  = new BriConWheel(this);
  m_briSlider->setMaximumHeight(15);
  m_conSlider->setMaximumHeight(15);
  
  QToolButton* resetBriCon = new QToolButton( this );
  resetBriCon->setPixmap(QPixmap(bricon));
  resetBriCon->setMaximumSize(QPixmap(bricon).size());
  
  QHBoxLayout* hlayout = new QHBoxLayout(this);
  hlayout->addWidget(m_minBox);
  hlayout->addWidget(resetBriCon);
  QVBoxLayout* slayout = new QVBoxLayout(hlayout);
  QHBoxLayout* blayout = new QHBoxLayout(slayout);

  blayout->addWidget(m_briSlider);
  QHBoxLayout* clayout = new QHBoxLayout(slayout);

  clayout->addWidget(m_conSlider);
  hlayout->addWidget(m_maxBox);

  QToolTip::add(m_minBox,tr("Minimum brightness value"));
  QToolTip::add(m_maxBox,tr("Maximum brightness value"));
  QToolTip::add(m_briSlider,tr("Adjust brightness"));
  QToolTip::add(m_conSlider,tr("Adjust contrast"));
  QToolTip::add(resetBriCon,tr("Reset contrast and brightness"));

  connect(resetBriCon, SIGNAL(pressed()),this, SLOT(reset())); 
  connect(m_minBox, SIGNAL(textChanged(const QString&)),this, SLOT(minChanged(const QString&))); 
  connect(m_maxBox, SIGNAL(textChanged(const QString&)),this, SLOT(maxChanged(const QString&))); 
  connect(m_briSlider, SIGNAL(valueChanged(double)),this, SLOT(briSliderChanged(double))); 
  connect(m_conSlider, SIGNAL(valueChanged(double)),this, SLOT(conSliderChanged(double)));
  connect(m_briSlider, SIGNAL(sliderReleased()), this, SLOT(sliderReleased()));
  connect(m_conSlider, SIGNAL(sliderReleased()), this, SLOT(sliderReleased()));
}

BriConWidget::~BriConWidget()
{
  m_list->detach(this);
}

void BriConWidget::reset()
{
  m_bricon->reset();
  sliderReleased();
}

void BriConWidget::attach(BriConWidgetObserver* o)
{
  m_observers.remove(o);
  m_observers.push_back(o);
}

void BriConWidget::detach(BriConWidgetObserver* o)
{
  m_observers.remove(o);
}

struct Update
{
  Update(BriConWidget* l): m_ol(l) {}

  void operator()(BriConWidgetObserver* v)
  {
    v->update(m_ol);
  }

  BriConWidget* m_ol;
};

void BriConWidget::notify()
{
  TRACKER("BriConWidget::notify");
  
  CHECKPOINT();
  if(!m_blockEvents)
    {
      m_blockEvents = true;
      std::for_each(m_observers.begin(), m_observers.end(), Update(this));
      m_blockEvents = false;
    }
}

void BriConWidget::update(const OverlayList* list, OverlayListMsg msg)
{
  m_blockEvents = true;

  if(OverlayListMsg(Select) == msg) 
    {
      MetaImage::Handle mi = list->getActiveMetaImage();
      if(!mi.get())
        mi = list->getMainMetaImage();
      m_bricon = mi->getDs()->inqBriCon();
      m_minBox->setText( QString("%1").arg(m_bricon->inqMin(),7,'g',5) );
      m_maxBox->setText( QString("%1").arg(m_bricon->inqMax(),7,'g',5) );
    }

  m_blockEvents = false;
}

void BriConWidget::minChanged(const QString& valueText)
{
  TRACKER("BriConWidget::minChanged");
  
  float value = valueText.toFloat();
  m_bricon->setMin(value);
  notify();
}

void BriConWidget::maxChanged(const QString& valueText)
{
  TRACKER("BriConWidget::maxChanged");
  float value = valueText.toFloat();
  m_bricon->setMax(value);
  notify();
}

void BriConWidget::briSliderChanged(double value)
{
  TRACKER("BriConWidget::briSliderChanged");

  m_bricon->modifyRange(value / 100.0, 0.0);
  updateMinMaxBoxes();
  notify();
}

void BriConWidget::conSliderChanged(double value)
{
  TRACKER("BriConWidget::conSliderChanged");
  m_bricon->modifyRange(0.0, value / 100.0);
  updateMinMaxBoxes();
  notify();
}

void BriConWidget::sliderReleased()
{
  TRACKER("BriConWidget::sliderReleased");

  m_bricon->updateRange();

  m_blockEvents = true;
  m_minBox->setText( QString("%1").arg(m_bricon->inqMin(),7,'g',5) );
  m_maxBox->setText( QString("%1").arg(m_bricon->inqMax(),7,'g',5) );
  m_blockEvents = false;
  
  m_briSlider->setValue(0);
  m_conSlider->setValue(0);

  notify();
}

void BriConWidget::updateMinMaxBoxes()
{
      m_minBox->blockSignals(true);
      m_minBox->setText( QString("%1").arg(m_bricon->inqAdjustedMin(),7,'g',5) );      
      m_minBox->blockSignals(false);

      m_maxBox->blockSignals(true);
      m_maxBox->setText( QString("%1").arg(m_bricon->inqAdjustedMax(),7,'g',5) );
      m_maxBox->blockSignals(false);
}
