/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#include "imagewidget.h"
#include "slicewidget.h"
#include <qwidget.h>
#include <qcheckbox.h>
#include <qtoolbar.h>
#include <qspinbox.h>
#include <qslider.h>
#include <qtooltip.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qtoolbutton.h>
#include <qbuttongroup.h>
#include <qmessagebox.h>
#include "overlaylist.h"
#include "overlaywidget.h"
#include "overlaydialog.h"
#include "briconwidget.h"
#include "drawwidget.h"
#include "cursorwidget.h"
#include <qmessagebox.h>
#include "movie.xpm"

#include "tracker.h"

#include "crosshair.xpm"
#include "zoom.xpm"
#include "pan.xpm"
#include "pen.xpm"
#include "reset.xpm"

const char * toggleCrossHairText = "Cross-hairs on/off.<br><hr>Hides/shows the cursor cross-hairs.";

const char * cursorText = "Cursor.<br><hr>Use this mode to "
"interogate the image. Click & drag to move the cursor.";
 
const char * zoomText = "Zoom.<br><hr>"
"Drag with the left mouse button to select the region to be zoomed up, "
"the ctrl-left mouse button will return the view to its previous state. "
"The right mouse button can also be used to zoom the image.";

const char * maskText = "Masking.<br><hr>"
"Edit images in this mode, images must by unlocked before they can be edited.";

const char * panText = "Pan.<br><hr>"
"Click and drag to achieve the desired view. The middle button can be also used to pan the image.";

const char * resetText = "Reset Zoom.<br><hr>"
"Use this button to reset images to their original position and zoom factor.";

static const char * movieText = "Movie mode.<br><hr>"
"Click to automatically move through time.";

ImageWidget::ImageWidget(QWidget* parent,        ImageGroup::Handle i,
                         OverlayList::Handle ol, Cursor::Handle c): 
                         ViewWidget(parent), m_imageGroup(i),
                         m_cursor(c),m_overlayList(ol)
{  
  constructor();
}


void ImageWidget::constructor()
{
  QFont f("Helvetica", 8);
  setFont( f );

  if(!isValidOverlayList(m_overlayList))
      m_overlayList = OverlayList::create(m_imageGroup);
  
  m_overlayDialog = NULL;
 
  setMinimumSize(210,345);
  constructToolBar();
  connectControls();  

  m_drawWidget->attach(this);
  m_overlayList->attach(this);
  m_cursor->attach(this);
}

ImageWidget::~ImageWidget()
{
  TRACKER("ImageWidget::~ImageWidget()");
  m_overlayWidget->close();
  m_drawWidget->detach(this);
  m_overlayList->detach(this); 
  m_cursor->detach(this);
  m_movieTimer->stop();

}

void ImageWidget::constructToolBar()
{
  QToolBar *tb1 = new QToolBar(this,"");
  QToolBar *tb2 = new QToolBar(this,"");
  QToolBar *tb3 = new QToolBar(this,"");
  QToolBar *tb4 = new QToolBar(this,"");
  QToolBar *tb5 = new QToolBar(this,"");

  m_zoomSpinBox = new QSpinBox(10, 10000, 10, tb1);
  m_zoomSpinBox->setSuffix("%");
  m_zoomSpinBox->setValue(100);
  m_zoomSpinBox->setButtonSymbols(QSpinBox::PlusMinus);
  QToolTip::add(m_zoomSpinBox, tr("Zoom factor"));

  ImageInfo::Handle i(m_imageGroup->getMainImage()->getInfo());

  m_volSpinBox = new QSpinBox(0, i->inqNumVolumes() - 1, 1, tb1);
  m_volSpinBox->setValue(0);
  m_volSpinBox->setButtonSymbols(QSpinBox::PlusMinus);
  QToolTip::add(m_volSpinBox, tr("Volume selection"));
  
  QPixmap cursorIcon = QPixmap( crosshair );
  m_cursorModeButton
    = new QToolButton( cursorIcon, tr( cursorText ), QString::null,
                       this, SLOT(setCursorMode()), tb1, "voxel select" );
  m_cursorModeButton->setToggleButton(true);

  QPixmap panIcon = QPixmap( pan_xpm );
  m_panModeButton
    = new QToolButton( panIcon, tr( panText ), QString::null,
                       this, SLOT(setPanMode()), tb1, "pan adjust" );
  m_panModeButton->setToggleButton(true);

  QPixmap zoomIcon = QPixmap( zoom );
  m_zoomModeButton
    = new QToolButton( zoomIcon, tr( zoomText ), QString::null,
                       this, SLOT(setZoomMode()), tb1, "zoom adjust" );
  m_zoomModeButton->setToggleButton(true);  

  QPixmap maskIcon = QPixmap( pen_xpm );
  m_maskModeButton
    = new QToolButton( maskIcon, tr( maskText ), QString::null,
                       this, SLOT(setMaskMode()), tb1, "masking mode" );
  connect(m_maskModeButton,SIGNAL(toggled(bool)),this,SLOT(maskButtonToggled(bool)));
  m_maskModeButton->setToggleButton(true);

  QButtonGroup* group = new QButtonGroup;
  group->insert(m_cursorModeButton);
  group->insert(m_zoomModeButton);
  group->insert(m_panModeButton);
  group->insert(m_maskModeButton);
  group->setExclusive(true);

  m_crossHairCheckBox = new QCheckBox( QString::null, tb1);
  m_crossHairCheckBox->setChecked(true);
  QToolTip::add(m_crossHairCheckBox, tr(toggleCrossHairText));
  connect( m_crossHairCheckBox, SIGNAL(stateChanged(int)), this, SLOT(setCrossHairMode()) );
  
  tb1->addSeparator();

  QPixmap resetIcon = QPixmap( reset_xpm );
  m_resetButton = new QToolButton( resetIcon,tr(resetText), QString::null,
                                   this,SIGNAL(resetZoom()), tb1, "reset zoom");

  m_briconWidget   = new BriConWidget(tb2,m_overlayList);
  m_overlayWidget  = new OverlayWidget(tb3,m_overlayList);
  m_cursorWidget   = new CursorWidget(tb4,m_cursor,m_overlayList);
  m_drawWidget     = new DrawWidget(tb5);
  m_drawWidget->setEnabled(false);

 
  connect(m_overlayWidget,SIGNAL(infoButtonAction()),
          this,           SLOT(openOverlayDialog()));

  addToolBar(tb1, Top,    FALSE);  
  addToolBar(tb2, Top,    FALSE);  
  addToolBar(tb4, Top,    TRUE); 
  addToolBar(tb3, Bottom, TRUE);
  addToolBar(tb5, Top,    FALSE);
 
  m_toolbar = tb1;  

  ImageInfo::Handle info(m_imageGroup->getMainImage()->getInfo());

  m_movieTimer = new QTimer(this);

  m_movieButton = new QToolButton(QPixmap(movie_xpm), tr(movieText),
                                                 QString::null, this, SLOT(toggleMovie()),
                                                 m_toolbar);
  m_movieButton->setToggleButton(true);

  connect( m_movieTimer, SIGNAL(timeout()), this, SLOT(nextFrame()) );
   
  if(info->inqNumVolumes() > 1)
    m_movieButton->setEnabled(true);
  else 
    m_movieButton->setEnabled(false);

  //m_speedSpinBox = new QSpinBox(10, 10000, 10, m_toolbar);
  //m_speedSpinBox->setValue(50);
  //m_speedSpinBox->setButtonSymbols(QSpinBox::PlusMinus);
  //QToolTip::add(m_speedSpinBox, tr("Zoom factor"));
  
}
  
void ImageWidget::connectControls()
{
  connect( m_volSpinBox,  SIGNAL(valueChanged(int)), 
           this,          SLOT  (volChanged(int)));
  connect( m_zoomSpinBox, SIGNAL(valueChanged(int)), 
           this,          SIGNAL(zoomChanged(int)));
}
  
void ImageWidget::setCrossHairMode()
{
  emit crossHairModeChanged(m_crossHairCheckBox->isChecked());
}

void ImageWidget::setPanMode() { emit modeChanged(SliceWidget::Pan); }

void ImageWidget::setCursorMode() { emit modeChanged(SliceWidget::Cursing); }

void ImageWidget::setZoomMode() { emit modeChanged(SliceWidget::Zoom); }

void ImageWidget::setMaskMode() { emit modeChanged(SliceWidget::Masking); }

void ImageWidget::volChanged(int n)
{
  TRACKER("ImageWidget::volChanged setCursor");
  m_cursor->setCursor(m_cursor->inqX(),m_cursor->inqY(),m_cursor->inqZ(),n);
}

void ImageWidget::setZoomCombo(int f)
{
  TRACKER("ImageWidget::setZoomCombo");
  blockSignals(true);
  m_zoomSpinBox->setValue(f);
  blockSignals(false);
}

void ImageWidget::setVolSpinBox(int v)
{
  TRACKER("ImageWidget::setVolSpinBox");
  m_volSpinBox->blockSignals(true);
  m_volSpinBox->setValue(v);
  m_volSpinBox->blockSignals(false);
}

void ImageWidget::openOverlayDialog()
{
  TRACKER("ImageWidget::openOverlayDialog");
  if(m_overlayDialog)delete m_overlayDialog;
  m_overlayDialog = new OverlayDialog(this,m_overlayList,m_imageGroup);
  m_overlayDialog->setSizePolicy(QSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed));
  m_overlayDialog->show();
   connect(m_overlayDialog,SIGNAL(openLookUpTable()),
           this,SIGNAL(addLookUpTable()));
   connect(m_overlayDialog,SIGNAL(message(const QString&, int)),
           this,SIGNAL(message(const QString&, int)));
}

void ImageWidget::update(const Cursor::Handle& c)
{
  setVolSpinBox(c->inqV());
}
  
void ImageWidget::update(const DrawWidget* w)
{
  if(w->inqUndoPressed())undoGraphics(); 
}

void ImageWidget::update(const OverlayList* i, OverlayListMsg msg)
{
  if(OverlayListMsg(Select) == msg){clearUndoList();}
  if(OverlayListMsg(DtiMode) == msg)
    {
      MetaImage::Handle m = i->getActiveMetaImage();
      if(m.get())
      {
        dtiDisplayMode(m->getDs()->inqDtiDisplay());
      }
    }
}  


void ImageWidget::clearUndoList()
{
  m_undoList.clear();
}


void ImageWidget::undoGraphics()
{
  if(!m_undoList.empty())
    {
      Shape::Handle undoBuffer = m_undoList.back();
      m_undoList.pop_back();
      undoBuffer->commit();
    } 
  m_cursor->repaint();
}

OverlayList::Handle ImageWidget::getOverlayList()
{
 return  m_overlayList;
}

void ImageWidget::maskButtonToggled(bool state)
{  
  if(state)  { m_drawWidget->setEnabled(true); }
  else       { m_drawWidget->setEnabled(false);}
}

void ImageWidget::nextFrame()
{
  ImageInfo::Handle info(m_imageGroup->getMainImage()->getInfo());
  int n = m_cursor->inqV();
  if(++n >= info->inqNumVolumes())n = 0;
 
  m_cursor->setCursor(m_cursor->inqX(), m_cursor->inqY(), m_cursor->inqZ(), n);  setVolSpinBox(n);
  
}

void ImageWidget::toggleMovie()
{
  if(!m_movieTimer->isActive())
    m_movieTimer->start(50, false);
  else
    m_movieTimer->stop();
}  

void ImageWidget::dtiDisplayMode(int dtiMode)
{
  if(DtiDisplay(None) == dtiMode)
    {  
      ImageInfo::Handle info(m_imageGroup->getMainImage()->getInfo());
      m_volSpinBox->setEnabled(true); 

      if(info->inqNumVolumes() > 1)
        m_movieButton->setEnabled(true);
      else 
        m_movieButton->setEnabled(false);
    }
  else
    { 
      m_volSpinBox->setValue(0);
      m_volSpinBox->setEnabled(false); 
      m_movieButton->setEnabled(false);  
      m_movieTimer->stop();
    }
}



