#include "tsplotcode.h"
#include "modelfit.h"
#include "tracker.h"
#include <math.h>


ModelFit::Handle ModelFit::create(const QString& fd)
{
  Handle dst(new ModelFit(fd));
  return dst;
}

ModelFit::ModelFit(const QString& fd):m_featDir(fd)
{
  readModel(m_featDir + "/design.mat");
  loadThresAc(m_featDir + "/stats/threshac1");
  loadFiltFuncData(m_featDir + "/filtered_func_data");
  loadPeData(m_featDir + "/stats/pe",
             m_designMatrix.Nrows(),
             m_designMatrix.Ncols());
 
}

ModelFit::~ModelFit(){}

void  ModelFit::readModel(const QString & filePath)
{
  FILE *fp;

  if((fp=fopen(filePath.latin1(),"r")))
    {
      int nevs = findKey(fp,"/NumWaves").toInt();
      int npts = findKey(fp,"/NumPoints").toInt();

      m_designMatrix.ReSize(npts,nevs);

      findKey(fp,"/Matrix");

      double dataVal;

      for(int j=1;j<=npts;j++)
        {
          for(int i=1;i<=nevs;i++)
            {
             fscanf(fp,"%lf",&dataVal);
             m_designMatrix(j,i) = dataVal;
            }
        }
      fclose(fp);
    }
  else
    {
      warning("failed to open file");
    }
}

QString ModelFit::findKey(FILE *fd, const QString & keyName )
{
  char charArray[100];
  QString valueStr;
  bool keyFound(false);
  
  fseek(fd,0,SEEK_SET);
  
  while(!keyFound && fgets(charArray, 100, fd))
    {
      QString lineStr(charArray);
      if(lineStr.find(keyName) != -1)
      {
        valueStr = lineStr.remove(0,keyName.length());
        keyFound = true;
      }
    }	  
  //file pointer left at pos just after key
  return valueStr;
}

void ModelFit::loadThresAc(const QString & filePath)
{  
  TRACKER("ModelFit::loadThresAc");

  m_threshAc = Image::load((const char *)filePath,false);
}

void ModelFit::loadPeData(const QString & filePath,int npts,int nevs)
{  
  TRACKER("ModelFit::loadPeData");

  Image::Handle peData;
  for(int n = 1; n <= nevs; ++n)
    {
      QString peFile = filePath + "1";
      peData = Image::load((const char *)peFile,false);
      m_peData.push_back(peData->getVolume(0));
    }
}

void ModelFit::loadFiltFuncData(const QString & filePath)
{  
  TRACKER("ModelFit::loadFiltFuncData");

  m_filtFuncData = Image::load((const char *)filePath,false);
}

ColumnVector ModelFit::getPeColumnVector(short x, short y, short z)
{  
  TRACKER("ModelFit::getPeColumnVector");

  int nevs = m_peData.size();
  ColumnVector peCv(nevs);

  for(int n = 0; n < nevs; ++n)
    {
      peCv(n+1) = m_peData[n]->value(x,y,z);
    }

  return peCv;
}

ColumnVector ModelFit::getThrColumnVector(short x, short y, short z)
{  
  TRACKER("ModelFit::getThrColumnVector");

  TimeSeries::Handle ts = m_threshAc->getTimeSeries(x,y,z);

  ColumnVector ac(ts->inqVolCount());
  
  for(int j = 0;j < ts->inqVolCount();++j)
    ac(j+1) = ts->real(j); 

  return ac;
}


TimeSeries::Handle ModelFit::getModelTimeSeries(short x, short y, 
                                                short z,float offset)
{
  TRACKER("ModelFit::getModelTimeSeries");
  
  ColumnVector ac(getThrColumnVector(x,y,z));
  
  Matrix model;

  TsPlotCode::preWhitenModel(ac,
                             m_designMatrix,model);
  
 
  ColumnVector peCv(getPeColumnVector(x,y,z));

  model *= peCv;  //full model

  TimeSeries::Handle modelTs = TimeSeriesD::create(x,y,z,
                                                  m_designMatrix.Nrows());

  for(int n = 0;n < model.Nrows();n++)
    modelTs->setValue(n,model(n+1,1) + offset);

  return modelTs;
}

TimeSeries::Handle ModelFit::getDataTimeSeries(short x, short y, short z)
{  
  TRACKER("ModelFit::getDataTimeSeries");
  ColumnVector ac(getThrColumnVector(x,y,z));
  TimeSeries::Handle ts = m_filtFuncData->getTimeSeries(x,y,z);
  return TsPlotCode::preWhitenTimeseries(ac,ts);
}
