/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#include "overlaydialog.h"
#include "imagedisplaysetting.h"
#include <qlabel.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qvalidator.h>
#include "fileopen.xpm"

class AddLutToCombo {
public:
  AddLutToCombo(QComboBox* c,LookUpTable::Handle l):m_cb(c),m_curLut(l),m_index(0)
  {
    m_cb->clear();
    m_cb->insertItem("Default");++m_index;
  }
  void operator()(LookUpTable::Handle l)
  { 
    m_cb->insertItem(l->inqLutName().c_str());  
    if(l == m_curLut){m_cb->setCurrentItem(m_index);}
    ++m_index;
  }
 
private:
  QComboBox* m_cb;
  LookUpTable::Handle m_curLut;
  int m_index;
};


class AddModImageToCombo {
public:
  AddModImageToCombo(QComboBox* c,Image::Handle i):
    m_cb(c),m_curModImg(i),m_index(0)
  {
    m_cb->clear();
    m_cb->insertItem("None");++m_index;
    
  }
  void operator()(Image::Handle i)
  { 
    m_cb->insertItem(i->getInfo()->inqImageName().c_str());  
    if(m_curModImg.get())
      {
        if(i == m_curModImg){m_cb->setCurrentItem(m_index);}
      }
    else
      {
        m_cb->setCurrentItem(0);
      }
    ++m_index;
  }
 
private:
  QComboBox* m_cb;
  Image::Handle m_curModImg;
  int m_index;
};



OverlayDialog::OverlayDialog(QWidget* w,OverlayList::Handle l,ImageGroup::Handle i):
  QDialog(w,0,FALSE), m_overlayList(l),m_imageGroup(i),m_blockOverlayListUpdate(false)
{  
  setCaption("Overlay Information");  
  QFont f("Helvetica", 10);
  setFont( f );
  m_overlayList->attach(this); 
  QPixmap fileIcon = QPixmap( fileopen_xpm );
  QIconSet fileIconSet = QIconSet(fileopen_xpm);
  

  QLabel* name       = new QLabel(this);
  QLabel* voxels     = new QLabel(this);
  QLabel* dimensions = new QLabel(this);
  QLabel* volume     = new QLabel(this);
  QLabel* bits       = new QLabel(this);
  QLabel* lut        = new QLabel(this);
  QLabel* dti        = new QLabel(this);
  QLabel* mod        = new QLabel(this);
  
  m_grid = new QGridLayout(this, 9, 3, 10, 5);

  m_grid->addWidget(name,0,0);
  m_grid->addWidget(voxels,1,0);
  m_grid->addWidget(dimensions,2,0);
  m_grid->addWidget(volume,3,0);
  m_grid->addWidget(bits,4,0);
  m_grid->addWidget(lut,5,0);
  m_grid->addWidget(dti,6,0);
  m_grid->addWidget(mod,7,0);

  name->setText("Name:");
  voxels->setText("Voxels:");
  dimensions->setText("Dimensions:");
  volume->setText("Volumes:");
  bits->setText("Bits/Pixel:");
  lut->setText("Look Up Table:");
  dti->setText("DTI Display:");
  mod->setText("Modulation:");


  m_overlayEdit = new QLineEdit(this);
  m_voxLabel = new QLabel(this); 
  m_dimLabel = new QLabel(this);
  m_volLabel = new QLabel(this);
  m_bitsLabel = new QLabel(this);
  m_lutCombo = new QComboBox(false,this);
  m_lutButton = new QPushButton(fileIcon,"",this);
  m_dtiCombo = new QComboBox(false,this); 
  m_modCombo = new QComboBox(false,this);
  m_okButton = new QPushButton("OK",this);

  m_grid->addWidget(m_overlayEdit,0,1);
  m_grid->addWidget(m_voxLabel,1,1);
  m_grid->addWidget(m_dimLabel,2,1);
  m_grid->addWidget(m_volLabel,3,1);
  m_grid->addWidget(m_bitsLabel,4,1);
  m_grid->addWidget(m_lutCombo,5,1);

  m_grid->addWidget(m_lutButton,5,2);
  m_grid->addWidget(m_dtiCombo,6,1);
  m_grid->addWidget(m_modCombo,7,1);
  m_grid->addWidget(m_okButton,8,2);

  setMinimumSize(m_grid->minimumSize());
  setFixedSize(m_grid->minimumSize());
  
  connect(m_lutCombo, SIGNAL(activated(int)),
          this,SLOT(lutComboChanged(int)));
  connect(m_lutButton, SIGNAL(pressed()),
          this,SLOT(lutButtonPressed()));  
  connect(m_dtiCombo, SIGNAL(activated(int)),
          this,SLOT(dtiComboChanged(int)));  
  connect(m_modCombo, SIGNAL(activated(int)),
          this,SLOT(modComboChanged(int)));
  connect(m_okButton, SIGNAL(pressed()),
          this,SLOT(okButtonPressed()));
  connect(m_overlayEdit,SIGNAL(textChanged(const QString &)),
          this,SLOT(overlayTextChanged(const QString &)));  

  updateDialog();
  QRegExp regexp("[a-zA-Z0-9_+.-]*");  

  #if (QT_VERSION >= 300)  
	m_nameValidator = new QRegExpValidator(regexp,this);
    m_overlayEdit->setValidator(m_nameValidator);
  #endif

   
  m_overlayEdit->setMaxLength(24);
  m_okButton->setFocus();
}

OverlayDialog::~OverlayDialog()
{  
  m_overlayList->detach(this);
}

void OverlayDialog::update(const OverlayList* l, OverlayListMsg msg)
{
  if(!m_blockOverlayListUpdate)
    updateDialog();
}

void OverlayDialog::updateDialog()
{   
  MetaImage::Handle metaImage = m_overlayList->getActiveMetaImage();

  if(!metaImage.get()) 
  {
    clearDialog();
  }
  else
  {
    ImageDisplaySetting::Handle disp = metaImage->getDs();
    ImageInfo::Handle           info = metaImage->getImage()->getInfo();
    LookUpTable::Handle         curLut = disp->inqLookUpTable();
    Image::Handle               curModImg = metaImage->getDs()->inqModImage();

    QString voxStr(tr("%1 x %2 x %3").arg(info->inqX())
                                       .arg(info->inqY())
                                       .arg(info->inqZ()));  
    QString dimStr(tr("%1 x %2 x %3 mm").arg(info->inqXDim())
                                       .arg(info->inqYDim())
                                       .arg(info->inqZDim()));
    QString volStr(tr("%1").arg(info->inqNumVolumes()));
    QString bitsStr(tr("%1").arg(info->inqBitsPerVoxel()));
   
    m_overlayEdit->blockSignals(true);
    m_overlayEdit->setText(info->inqImageName().c_str());   
    m_overlayEdit->blockSignals(false);
    m_voxLabel->setText(voxStr);
    m_dimLabel->setText(dimStr);
    m_volLabel->setText(volStr);
    m_bitsLabel->setText(bitsStr);
    
    std::for_each(m_imageGroup->beginLutList(),
                  m_imageGroup->endLutList(),
                  AddLutToCombo(m_lutCombo,curLut));
    
    std::for_each(m_imageGroup->begin(),
                  m_imageGroup->end(),
                  AddModImageToCombo(m_modCombo,curModImg));

    m_dtiCombo->clear();
    m_dtiCombo->insertItem("None");
    m_dtiCombo->insertItem("Lines");
    m_dtiCombo->insertItem("RGB");

    MetaImage::Handle metaImage = m_overlayList->getActiveMetaImage();

    if(metaImage.get()) 
      {            
        m_dtiCombo->blockSignals(true);
        int mode = metaImage->getDs()->inqDtiDisplay();
        switch (mode)
          {
          case DtiDisplay(None):
            m_dtiCombo->setCurrentItem(0);
            m_lutCombo->setEnabled(true);
            m_modCombo->setEnabled(false);
            break;
          case DtiDisplay(Lines):
            m_dtiCombo->setCurrentItem(1);           
            m_lutCombo->setEnabled(false);
            m_modCombo->setEnabled(false);
            break;
          case DtiDisplay(RGB):
            m_dtiCombo->setCurrentItem(2);            
            m_lutCombo->setEnabled(false);
            m_modCombo->setEnabled(true);
            break;
          }
         m_dtiCombo->blockSignals(false);   
      }
    
  }
}

void OverlayDialog::clearDialog()
{
  m_overlayEdit->setText("");
  m_lutCombo->clear();
  m_voxLabel->clear();
  m_dimLabel->clear();
  m_volLabel->clear();
  m_bitsLabel->clear();
  m_dtiCombo->clear();
  m_modCombo->clear();
}

void OverlayDialog::modComboChanged(int n)
{  
  Image::Handle img;
  
  if(n){img = m_imageGroup->getImage(n - 1);}
  m_overlayList->setModImage(img);
}

void OverlayDialog::lutComboChanged(int n)
{
  LookUpTable::Handle lut;
  
  if(n){lut = m_imageGroup->getLut(n - 1);}
  m_overlayList->setLookUpTable(lut);
}

void OverlayDialog::dtiComboChanged(int n)
{
  MetaImage::Handle metaImage = m_overlayList->getActiveMetaImage();

  if(metaImage.get()) 
    {
      if (metaImage->getInfo()->isDtiCompatible())
        {
          switch(n)
            {
            case 0:
              metaImage->getDs()->setDtiDisplay(DtiDisplay(None));
              m_lutCombo->setEnabled(true);
              m_modCombo->setEnabled(false);
              m_overlayList->notify(OverlayListMsg(DtiMode));
              break;
            case 1:
              metaImage->getDs()->setDtiDisplay(DtiDisplay(Lines)); 
              m_lutCombo->setEnabled(false);              
              m_modCombo->setEnabled(true);
              m_overlayList->notify(OverlayListMsg(DtiMode));
              break;
            case 2:   
              metaImage->getDs()->setDtiDisplay(DtiDisplay(RGB));
              m_lutCombo->setEnabled(false);              
              m_modCombo->setEnabled(true);
              m_overlayList->notify(OverlayListMsg(LookUpTable));
              break;
            }
        }
      else
        {
          emit message("Warning: Image is not a valid DTI image.",2000);
          m_dtiCombo->blockSignals(true);
          m_dtiCombo->setCurrentItem(0);   
          m_dtiCombo->blockSignals(false);
        }
    }
}

void OverlayDialog::lutButtonPressed()
{
  openLookUpTable();
}

void OverlayDialog::okButtonPressed()
{
  close();
}

void OverlayDialog::overlayTextChanged( const QString & newName ) 
{ 
  MetaImage::Handle metaImage = m_overlayList->getActiveMetaImage();

  if(metaImage.get() && !newName.isEmpty()) 
    {
      if (metaImage->inqReadOnly())
        {
          emit message("Warning: Image must be unlocked before it can be renamed.",2000);    
          m_overlayEdit->blockSignals(true);   
          m_overlayEdit->setText(metaImage->getImage()->getInfo()->inqImageName().c_str());   
          m_overlayEdit->blockSignals(false);
          
        }
      else
        {metaImage->setImageName(newName.latin1());}
      m_blockOverlayListUpdate = true;
      m_imageGroup->notify(ImageGroupMsg(NameChange));
      m_blockOverlayListUpdate = false;
    }

}

