/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */


#include "timeserieswidget.h"
#include "cursor.h"
#include "tracker.h"
#include "storage/timeseries.h"
#include <qlayout.h>
#include <qpushbutton.h>
#include <qtoolbar.h>
#include <qspinbox.h>
#include <qbuttongroup.h>
#include <qtoolbutton.h>
#include <qtooltip.h>
#include <list>

#include "add.xpm"
#include "rem.xpm"
#include "demean.xpm"
#include "axis.xpm"

#include "singleseries.xpm"
#include "gridseries.xpm"
#include "cubeseries.xpm"



#include "singleserieswidget.h"
#include "gridserieswidget.h"
#include "cubeserieswidget.h"

#include "qwt_plot.h"

const char * addSeriesText = "Add series.<br><hr>Press to add the timeseries of the current cursor position.";

const char * remSeriesText = "Remove series.<br><hr>Press to remove the currently selected time series.";

const char * demeanText = "Demeaning.<br><hr>Subtracts the timeseries mean from all plot values.";

const char * axisText = "Suppress Axis.<br><hr>Disable axis and grid display on timeseries graphs";

const char * singleViewText = "Single View.<br><hr>View a single timeseries graph window";

const char * gridViewText = "Grid View.<br><hr>View a grid of timeseries windows";

const char * cubeViewText = "Cube View.<br><hr>View a cube layout of timeseries windows";

TimeSeriesWidget::TimeSeriesWidget(QWidget *parent, 
                                   Image::Handle& image, 
                                   Cursor::Handle& cursor):
  ViewWidget(parent),m_image(image),
  m_cursor(cursor),m_viewNumber(0)
{  
  m_options = PlotOptions::create();

  constructor();
}

TimeSeriesWidget::TimeSeriesWidget(QWidget *parent, 
                                   Image::Handle& image, 
                                   Cursor::Handle& cursor,
                                   ModelFit::Handle& modelFit):
  ViewWidget(parent),m_image(image),
  m_cursor(cursor),m_viewNumber(0)
{  
  m_options = PlotOptions::create();
  m_options->setModelFit(modelFit);

  constructor();
}

void TimeSeriesWidget::constructor()
{  
  TRACKER("TimeSeriesWidget::constructor");  

  QPixmap addIcon = QPixmap( add );
  QPixmap remIcon = QPixmap( rem );
  QPixmap demIcon = QPixmap( demean );
  QPixmap axisIcon = QPixmap( axis );
  
  QPixmap singleIcon = QPixmap( singleseries );
  QPixmap gridIcon =   QPixmap( gridseries );
  QPixmap cubeIcon =   QPixmap( cubeseries );




  QToolBar *tb1 = new QToolBar(this, "");  
 

  m_addPlot = new QToolButton( addIcon, tr( addSeriesText ), 
                               QString::null,NULL, 
                               NULL, 
                               tb1, "Add time series" );
  m_remPlot = new QToolButton( remIcon, tr( remSeriesText ), 
                               QString::null,NULL, 
                               NULL, 
                               tb1, "Remove time series" );  

  m_demeanButton = new QToolButton( demIcon, tr( demeanText ), 
                                   QString::null, NULL,
                                   NULL, 
                                   tb1, "Demean follows cursor" ); 
  m_demeanButton->setToggleButton(true);

  m_axisButton = new QToolButton( axisIcon, tr( axisText ), 
                                   QString::null, NULL,
                                   NULL, 
                                   tb1, "Suppress axis" ); 


  tb1->addSeparator();

  m_singleButton = new QToolButton( singleIcon, tr( singleViewText ), 
                                   QString::null, this,
                                   SLOT(singleView()), 
                                   tb1, "Single timeseries view" );  

  m_singleButton->setToggleButton(true);
  m_singleButton->setOn(true);

  m_gridButton = new QToolButton( gridIcon, tr( gridViewText ), 
                                   QString::null, this,
                                   SLOT(gridView()), 
                                   tb1, "Grid of timeseries views" );  

  m_gridButton->setToggleButton(true);

  m_cubeButton = new QToolButton( cubeIcon, tr( cubeViewText ), 
                                   QString::null, this,
                                   SLOT(cubeView()), 
                                   tb1, "Cube of timeseries views" );  

  m_cubeButton->setToggleButton(true);

  QButtonGroup* group = new QButtonGroup;
  group->insert(m_singleButton);
  group->insert(m_gridButton);
  group->insert(m_cubeButton);
  group->setExclusive(true);



  newSlice();

  addToolBar(tb1,Top,FALSE);
}

TimeSeriesWidget::~TimeSeriesWidget()
{
};


void TimeSeriesWidget::singleView()
{
    m_viewNumber = TimeSeriesWidget::Single;
    newSlice();
}

void TimeSeriesWidget::gridView()
{
    m_viewNumber = TimeSeriesWidget::Grid;    
    newSlice();
}

void TimeSeriesWidget::cubeView()
{
    m_viewNumber = TimeSeriesWidget::Cube; 
    newSlice();
}


void TimeSeriesWidget::newSlice()
{
  TRACKER("TimeSeriesWidget::newSlice");

  setCentralWidget(NULL);

  if(m_displayWidget.get())
    {
    m_displayWidget->hide();  
    m_displayWidget->setEnabled(false);
    
 
    disconnect(m_demeanButton,SIGNAL(toggled(bool)),
               m_displayWidget.get(),SLOT(demeanButtonToggle(bool)));
    disconnect(m_addPlot,SIGNAL(pressed()),
               m_displayWidget.get(),SLOT(addTimeSeries()));
    disconnect(m_remPlot,SIGNAL(pressed()),
               m_displayWidget.get(),SLOT(remTimeSeries()));
    disconnect(this,SIGNAL(setEnabled(bool)),
               m_displayWidget.get(),SLOT(setEnabled(bool)));
    disconnect(m_axisButton,SIGNAL(pressed()),
               m_displayWidget.get(),SLOT(axisDisplay()));
    }


  switch(m_viewNumber)
    {
    case TimeSeriesWidget::Single:

      if(!m_singleWidget.get())
        {
          m_singleWidget =  TimeSeriesDisplay::Handle
        (new SingleSeriesWidget(this,m_image,m_cursor,m_options));
        }
      m_displayWidget = m_singleWidget;
      m_addPlot->setEnabled(m_options->inqAddRemEnabled());
      m_remPlot->setEnabled(m_options->inqAddRemEnabled());      
      m_demeanButton->setEnabled(true);
      break;
    
    case TimeSeriesWidget::Grid:

      if(!m_gridWidget.get())
        {       
          m_gridWidget =  TimeSeriesDisplay::Handle
        (new GridSeriesWidget(this,m_image,m_cursor,m_options,0)); 
        } 

      m_displayWidget = m_gridWidget;
      m_addPlot->setEnabled(false);      
      m_remPlot->setEnabled(false);
      m_demeanButton->setEnabled(false);
      break;
  
    case TimeSeriesWidget::Cube:
      if(!m_cubeWidget.get())
        {      
          m_cubeWidget = TimeSeriesDisplay::Handle
        (new CubeSeriesWidget(this,m_image,m_cursor,m_options));
        }
      m_displayWidget = m_cubeWidget;
      m_addPlot->setEnabled(false);      
      m_remPlot->setEnabled(false);
      m_demeanButton->setEnabled(false);      
      break;
    }
  
  connect(m_demeanButton,SIGNAL(toggled(bool)),
          m_displayWidget.get(),SLOT(demeanButtonToggle(bool)));
  connect(m_addPlot,SIGNAL(pressed()),
          m_displayWidget.get(),SLOT(addTimeSeries()));
  connect(m_remPlot,SIGNAL(pressed()),
          m_displayWidget.get(),SLOT(remTimeSeries()));
  connect(this,SIGNAL(setEnabled(bool)),
          m_displayWidget.get(),SLOT(setEnabled(bool)));
  connect(m_axisButton,SIGNAL(pressed()),
          m_displayWidget.get(),SLOT(axisDisplay()));

  m_displayWidget->setEnabled(true);

  setCentralWidget(m_displayWidget.get());  
  m_displayWidget->show();
  m_cursor->setCursor(m_cursor->inqX(),m_cursor->inqY(),m_cursor->inqZ());
}


